@@ -8,6 +8,7 @@ use ansi_term::{
     Style,
 };
 
+use std::collections::BTreeMap;
 use std::fmt::{Display, Error, Formatter};
 use zellij_tile::prelude::actions::Action;
 use zellij_tile::prelude::*;
@@ -176,7 +177,7 @@ fn color_elements(palette: Palette, different_color_alternates: bool) -> Colored
 }
 
 impl ZellijPlugin for State {
-    fn load(&mut self) {
+    fn load(&mut self, _configuration: BTreeMap<String, String>) {
         // TODO: Should be able to choose whether to use the cache through config.
         self.tip_name = get_cached_tip_name();
         set_selectable(false);
