@@ -621,11 +621,12 @@ pub(crate) fn route_action(
                 .send_to_screen(ScreenInstruction::StartOrReloadPluginPane(run_plugin, None))
                 .with_context(err_context)?;
         },
-        Action::LaunchOrFocusPlugin(run_plugin, should_float) => {
+        Action::LaunchOrFocusPlugin(run_plugin, should_float, move_to_focused_tab) => {
             senders
                 .send_to_screen(ScreenInstruction::LaunchOrFocusPlugin(
                     run_plugin,
                     should_float,
+                    move_to_focused_tab,
                     client_id,
                 ))
                 .with_context(err_context)?;
@@ -690,6 +691,25 @@ pub(crate) fn route_action(
                 ))
                 .with_context(err_context)?;
         },
+        Action::BreakPane => {
+            senders
+                .send_to_screen(ScreenInstruction::BreakPane(
+                    default_layout.clone(),
+                    default_shell.clone(),
+                    client_id,
+                ))
+                .with_context(err_context)?;
+        },
+        Action::BreakPaneRight => {
+            senders
+                .send_to_screen(ScreenInstruction::BreakPaneRight(client_id))
+                .with_context(err_context)?;
+        },
+        Action::BreakPaneLeft => {
+            senders
+                .send_to_screen(ScreenInstruction::BreakPaneLeft(client_id))
+                .with_context(err_context)?;
+        },
     }
     Ok(should_break)
 }
@@ -841,9 +861,19 @@ pub(crate) fn route_thread_main(
                                 .send(new_client_instruction)
                                 .with_context(err_context)?;
                         },
-                        ClientToServerMsg::AttachClient(client_attributes, opts) => {
-                            let attach_client_instruction =
-                                ServerInstruction::AttachClient(client_attributes, opts, client_id);
+                        ClientToServerMsg::AttachClient(
+                            client_attributes,
+                            opts,
+                            tab_position_to_focus,
+                            pane_id_to_focus,
+                        ) => {
+                            let attach_client_instruction = ServerInstruction::AttachClient(
+                                client_attributes,
+                                opts,
+                                tab_position_to_focus,
+                                pane_id_to_focus,
+                                client_id,
+                            );
                             to_server
                                 .send(attach_client_instruction)
                                 .with_context(err_context)?;
