@@ -18,7 +18,9 @@ use std::collections::{BTreeMap, HashMap, HashSet};
 use std::rc::Rc;
 use zellij_utils::{
     data::{Palette, Style},
-    input::layout::{FloatingPaneLayout, Run, RunPluginLocation, TiledPaneLayout},
+    input::layout::{
+        FloatingPaneLayout, PluginUserConfiguration, Run, RunPluginLocation, TiledPaneLayout,
+    },
     pane_size::{Offset, PaneGeom, Size, SizeInPixels, Viewport},
 };
 
@@ -96,7 +98,7 @@ impl<'a> LayoutApplier<'a> {
         floating_panes_layout: Vec<FloatingPaneLayout>,
         new_terminal_ids: Vec<(u32, HoldForCommand)>,
         new_floating_terminal_ids: Vec<(u32, HoldForCommand)>,
-        mut new_plugin_ids: HashMap<RunPluginLocation, Vec<u32>>,
+        mut new_plugin_ids: HashMap<(RunPluginLocation, PluginUserConfiguration), Vec<u32>>,
         client_id: ClientId,
     ) -> Result<bool> {
         // true => layout has floating panes
@@ -180,6 +182,11 @@ impl<'a> LayoutApplier<'a> {
                     }
                 }
                 pane_focuser.focus_tiled_pane(&mut self.tiled_panes);
+                LayoutApplier::offset_viewport(
+                    self.viewport.clone(),
+                    self.tiled_panes,
+                    self.draw_pane_frames,
+                );
             },
             Err(e) => {
                 Err::<(), _>(anyError::msg(e))
@@ -193,13 +200,14 @@ impl<'a> LayoutApplier<'a> {
         &mut self,
         layout: TiledPaneLayout,
         new_terminal_ids: Vec<(u32, HoldForCommand)>,
-        new_plugin_ids: &mut HashMap<RunPluginLocation, Vec<u32>>,
+        new_plugin_ids: &mut HashMap<(RunPluginLocation, PluginUserConfiguration), Vec<u32>>,
         client_id: ClientId,
     ) -> Result<()> {
         let err_context = || format!("failed to apply tiled panes layout");
         let free_space = self.total_space_for_tiled_panes();
         match layout.position_panes_in_space(&free_space, None) {
             Ok(positions_in_layout) => {
+                let mut run_instructions_to_ignore = layout.run_instructions_to_ignore.clone();
                 let positions_and_size = positions_in_layout.iter();
                 let mut new_terminal_ids = new_terminal_ids.iter();
 
@@ -211,11 +219,20 @@ impl<'a> LayoutApplier<'a> {
                 };
 
                 for (layout, position_and_size) in positions_and_size {
-                    // A plugin pane
-                    if let Some(Run::Plugin(run)) = layout.run.clone() {
+                    if let Some(position) = run_instructions_to_ignore
+                        .iter()
+                        .position(|r| r == &layout.run)
+                    {
+                        let run = run_instructions_to_ignore.remove(position);
+                        self.tiled_panes.set_geom_for_pane_with_run(
+                            run,
+                            *position_and_size,
+                            layout.borderless,
+                        );
+                    } else if let Some(Run::Plugin(run)) = layout.run.clone() {
                         let pane_title = run.location.to_string();
                         let pid = new_plugin_ids
-                            .get_mut(&run.location)
+                            .get_mut(&(run.location, run.configuration))
                             .and_then(|ids| ids.pop())
                             .with_context(err_context)?;
                         let mut new_plugin = PluginPane::new(
@@ -309,7 +326,7 @@ impl<'a> LayoutApplier<'a> {
         &mut self,
         floating_panes_layout: Vec<FloatingPaneLayout>,
         new_floating_terminal_ids: Vec<(u32, HoldForCommand)>,
-        new_plugin_ids: &mut HashMap<RunPluginLocation, Vec<u32>>,
+        new_plugin_ids: &mut HashMap<(RunPluginLocation, PluginUserConfiguration), Vec<u32>>,
         layout_name: Option<String>,
     ) -> Result<bool> {
         // true => has floating panes
@@ -320,13 +337,18 @@ impl<'a> LayoutApplier<'a> {
         let mut new_floating_terminal_ids = new_floating_terminal_ids.iter();
         for floating_pane_layout in floating_panes_layout {
             layout_has_floating_panes = true;
-            if let Some(Run::Plugin(run)) = floating_pane_layout.run.clone() {
-                let position_and_size = self
-                    .floating_panes
-                    .position_floating_pane_layout(&floating_pane_layout);
+            let position_and_size = self
+                .floating_panes
+                .position_floating_pane_layout(&floating_pane_layout);
+            if floating_pane_layout.already_running {
+                self.floating_panes.set_geom_for_pane_with_run(
+                    floating_pane_layout.run.clone(),
+                    position_and_size,
+                );
+            } else if let Some(Run::Plugin(run)) = floating_pane_layout.run.clone() {
                 let pane_title = run.location.to_string();
                 let pid = new_plugin_ids
-                    .get_mut(&run.location)
+                    .get_mut(&(run.location, run.configuration))
                     .and_then(|ids| ids.pop())
                     .with_context(err_context)?;
                 let mut new_pane = PluginPane::new(
@@ -363,9 +385,6 @@ impl<'a> LayoutApplier<'a> {
                     focused_floating_pane = Some(PaneId::Plugin(pid));
                 }
             } else if let Some((pid, hold_for_command)) = new_floating_terminal_ids.next() {
-                let position_and_size = self
-                    .floating_panes
-                    .position_floating_pane_layout(&floating_pane_layout);
                 let next_terminal_position =
                     get_next_terminal_position(&self.tiled_panes, &self.floating_panes);
                 let initial_title = match &floating_pane_layout.run {
@@ -504,28 +523,43 @@ impl<'a> LayoutApplier<'a> {
         self.tiled_panes.resize(new_screen_size);
         Ok(())
     }
-    fn offset_viewport(&mut self, position_and_size: &Viewport) {
-        let mut viewport = self.viewport.borrow_mut();
-        if position_and_size.x == viewport.x
-            && position_and_size.x + position_and_size.cols == viewport.x + viewport.cols
+    pub fn offset_viewport(
+        viewport: Rc<RefCell<Viewport>>,
+        tiled_panes: &mut TiledPanes,
+        draw_pane_frames: bool,
+    ) {
+        let boundary_geoms = tiled_panes.non_selectable_pane_geoms_inside_viewport();
         {
-            if position_and_size.y == viewport.y {
-                viewport.y += position_and_size.rows;
-                viewport.rows -= position_and_size.rows;
-            } else if position_and_size.y + position_and_size.rows == viewport.y + viewport.rows {
-                viewport.rows -= position_and_size.rows;
-            }
-        }
-        if position_and_size.y == viewport.y
-            && position_and_size.y + position_and_size.rows == viewport.y + viewport.rows
-        {
-            if position_and_size.x == viewport.x {
-                viewport.x += position_and_size.cols;
-                viewport.cols -= position_and_size.cols;
-            } else if position_and_size.x + position_and_size.cols == viewport.x + viewport.cols {
-                viewport.cols -= position_and_size.cols;
+            // curly braces here is so that we free viewport immediately when we're done
+            let mut viewport = viewport.borrow_mut();
+            for position_and_size in boundary_geoms {
+                if position_and_size.x == viewport.x
+                    && position_and_size.x + position_and_size.cols == viewport.x + viewport.cols
+                {
+                    if position_and_size.y == viewport.y {
+                        viewport.y += position_and_size.rows;
+                        viewport.rows -= position_and_size.rows;
+                    } else if position_and_size.y + position_and_size.rows
+                        == viewport.y + viewport.rows
+                    {
+                        viewport.rows -= position_and_size.rows;
+                    }
+                }
+                if position_and_size.y == viewport.y
+                    && position_and_size.y + position_and_size.rows == viewport.y + viewport.rows
+                {
+                    if position_and_size.x == viewport.x {
+                        viewport.x += position_and_size.cols;
+                        viewport.cols -= position_and_size.cols;
+                    } else if position_and_size.x + position_and_size.cols
+                        == viewport.x + viewport.cols
+                    {
+                        viewport.cols -= position_and_size.cols;
+                    }
+                }
             }
         }
+        tiled_panes.set_pane_frames(draw_pane_frames);
     }
     fn adjust_viewport(&mut self) -> Result<()> {
         // here we offset the viewport after applying a tiled panes layout
@@ -541,11 +575,11 @@ impl<'a> LayoutApplier<'a> {
             *display_area
         };
         self.resize_whole_tab(display_area).context(err_context)?;
-        let boundary_geoms = self.tiled_panes.borderless_pane_geoms();
-        for geom in boundary_geoms {
-            self.offset_viewport(&geom)
-        }
-        self.tiled_panes.set_pane_frames(self.draw_pane_frames);
+        LayoutApplier::offset_viewport(
+            self.viewport.clone(),
+            self.tiled_panes,
+            self.draw_pane_frames,
+        );
         Ok(())
     }
     fn set_focused_tiled_pane(&mut self, focus_pane_id: Option<PaneId>, client_id: ClientId) {
