@@ -10,6 +10,19 @@ pub enum TerminalAction {
     RunCommand(RunCommand),
 }
 
+impl TerminalAction {
+    pub fn change_cwd(&mut self, new_cwd: PathBuf) {
+        match self {
+            TerminalAction::OpenFile(_, _, cwd) => {
+                *cwd = Some(new_cwd);
+            },
+            TerminalAction::RunCommand(run_command) => {
+                run_command.cwd = Some(new_cwd);
+            },
+        }
+    }
+}
+
 #[derive(Clone, Debug, Deserialize, Default, Serialize, PartialEq, Eq)]
 pub struct RunCommand {
     #[serde(alias = "cmd")]
@@ -68,3 +81,29 @@ impl From<RunCommandAction> for RunCommand {
         }
     }
 }
+
+impl From<RunCommand> for RunCommandAction {
+    fn from(run_command: RunCommand) -> Self {
+        RunCommandAction {
+            command: run_command.command,
+            args: run_command.args,
+            cwd: run_command.cwd,
+            direction: None,
+            hold_on_close: run_command.hold_on_close,
+            hold_on_start: run_command.hold_on_start,
+        }
+    }
+}
+
+impl RunCommand {
+    pub fn new(command: PathBuf) -> Self {
+        RunCommand {
+            command,
+            ..Default::default()
+        }
+    }
+    pub fn with_cwd(mut self, cwd: PathBuf) -> Self {
+        self.cwd = Some(cwd);
+        self
+    }
+}
