@@ -207,11 +207,70 @@ impl Run {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
+#[allow(clippy::derive_hash_xor_eq)]
+#[derive(Debug, Serialize, Deserialize, Clone, Hash, Default)]
 pub struct RunPlugin {
     #[serde(default)]
     pub _allow_exec_host_cmd: bool,
     pub location: RunPluginLocation,
+    pub configuration: PluginUserConfiguration,
+}
+
+impl RunPlugin {
+    pub fn from_url(url: &str) -> Result<Self, PluginsConfigError> {
+        let location = RunPluginLocation::parse(url, None)?;
+        Ok(RunPlugin {
+            location,
+            ..Default::default()
+        })
+    }
+}
+
+#[allow(clippy::derive_hash_xor_eq)]
+impl PartialEq for RunPlugin {
+    fn eq(&self, other: &Self) -> bool {
+        // TODO: normalize paths here if the location is a file so that relative/absolute paths
+        // will work properly
+        (&self.location, &self.configuration) == (&other.location, &other.configuration)
+    }
+}
+impl Eq for RunPlugin {}
+
+#[derive(Debug, Default, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
+pub struct PluginUserConfiguration(BTreeMap<String, String>);
+
+impl PluginUserConfiguration {
+    pub fn new(mut configuration: BTreeMap<String, String>) -> Self {
+        // reserved words
+        configuration.remove("hold_on_close");
+        configuration.remove("hold_on_start");
+        configuration.remove("cwd");
+        configuration.remove("name");
+        configuration.remove("direction");
+        configuration.remove("floating");
+        configuration.remove("move_to_focused_tab");
+
+        PluginUserConfiguration(configuration)
+    }
+    pub fn inner(&self) -> &BTreeMap<String, String> {
+        &self.0
+    }
+}
+
+impl FromStr for PluginUserConfiguration {
+    type Err = &'static str;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let mut ret = BTreeMap::new();
+        let configs = s.split(',');
+        for config in configs {
+            let mut config = config.split('=');
+            let key = config.next().ok_or("invalid configuration key")?.to_owned();
+            let value = config.map(|c| c.to_owned()).collect::<Vec<_>>().join("=");
+            ret.insert(key, value);
+        }
+        Ok(PluginUserConfiguration(ret))
+    }
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq, Eq, Hash)]
@@ -220,6 +279,12 @@ pub enum RunPluginLocation {
     Zellij(PluginTag),
 }
 
+impl Default for RunPluginLocation {
+    fn default() -> Self {
+        RunPluginLocation::File(Default::default())
+    }
+}
+
 impl RunPluginLocation {
     pub fn parse(location: &str, cwd: Option<PathBuf>) -> Result<Self, PluginsConfigError> {
         let url = Url::parse(location)?;
@@ -372,6 +437,7 @@ pub struct FloatingPaneLayout {
     pub y: Option<PercentOrFixed>,
     pub run: Option<Run>,
     pub focus: Option<bool>,
+    pub already_running: bool,
 }
 
 impl FloatingPaneLayout {
@@ -409,6 +475,7 @@ pub struct TiledPaneLayout {
     pub children_are_stacked: bool,
     pub is_expanded_in_stack: bool,
     pub exclude_from_sync: Option<bool>,
+    pub run_instructions_to_ignore: Vec<Option<Run>>,
 }
 
 impl TiledPaneLayout {
@@ -530,8 +597,36 @@ impl TiledPaneLayout {
             let mut child_run_instructions = child.extract_run_instructions();
             run_instructions.append(&mut child_run_instructions);
         }
+        let mut successfully_ignored = 0;
+        for instruction_to_ignore in &self.run_instructions_to_ignore {
+            if let Some(position) = run_instructions
+                .iter()
+                .position(|i| i == instruction_to_ignore)
+            {
+                run_instructions.remove(position);
+                successfully_ignored += 1;
+            }
+        }
+        // we need to do this because if we have an ignored instruction that does not match any
+        // running instruction, we'll have an extra pane and our state will be messed up and we'll
+        // crash (this can happen for example when breaking a plugin pane into a new tab that does
+        // not have room for it but has a terminal instead)
+        if successfully_ignored < self.run_instructions_to_ignore.len() {
+            for _ in 0..self
+                .run_instructions_to_ignore
+                .len()
+                .saturating_sub(successfully_ignored)
+            {
+                if let Some(position) = run_instructions.iter().position(|i| i == &None) {
+                    run_instructions.remove(position);
+                }
+            }
+        }
         run_instructions
     }
+    pub fn ignore_run_instruction(&mut self, run_instruction: Option<Run>) {
+        self.run_instructions_to_ignore.push(run_instruction);
+    }
     pub fn with_one_pane() -> Self {
         let mut default_layout = TiledPaneLayout::default();
         default_layout.children = vec![TiledPaneLayout::default()];
